@@ -17,18 +17,18 @@ using namespace enigma2::data;
 using namespace enigma2::extract;
 using namespace enigma2::utilities;
 
-EpgEntryExtractor::EpgEntryExtractor()
-  : IExtractor()
+EpgEntryExtractor::EpgEntryExtractor(const std::shared_ptr<enigma2::InstanceSettings>& settings)
+  : IExtractor(settings)
 {
   FileUtils::CopyDirectory(FileUtils::GetResourceDataPath() + GENRE_DIR, GENRE_ADDON_DATA_BASE_DIR, true);
   FileUtils::CopyDirectory(FileUtils::GetResourceDataPath() + SHOW_INFO_DIR, SHOW_INFO_ADDON_DATA_BASE_DIR, true);
 
-  if (Settings::GetInstance().GetMapGenreIds())
-    m_extractors.emplace_back(new GenreIdMapper());
-  if (Settings::GetInstance().GetMapRytecTextGenres())
-    m_extractors.emplace_back(new GenreRytecTextMapper());
-  if (Settings::GetInstance().GetExtractShowInfo())
-    m_extractors.emplace_back(new ShowInfoExtractor());
+  if (m_settings->GetMapGenreIds())
+    m_extractors.emplace_back(new GenreIdMapper(m_settings));
+  if (m_settings->GetMapRytecTextGenres())
+    m_extractors.emplace_back(new GenreRytecTextMapper(m_settings));
+  if (m_settings->GetExtractShowInfo())
+    m_extractors.emplace_back(new ShowInfoExtractor(m_settings));
 
   m_anyExtractorEnabled = false;
   for (auto& extractor : m_extractors)
